# Adminer script discovery

include("compat.inc");

if(description)
{
    script_id(10797107);
    script_version("$Revision: 0.1 $");
    script_cvs_date("$Date: 2019/04/10 13:37:00 $");

    script_name(english: "Adminer Database Management Detection");

    script_set_attribute(attribute:"synopsis", value: "Adminer Database Management script was detected on the remote host.");
    script_set_attribute(attribute:"description", value: "Adminer Database Management script was detected on the remote host.");
    script_set_attribute(attribute:"solution", value: "Restrict access to the script, if desired.");
    script_set_attribute(attribute:"risk_factor", value: "Low");

    script_set_attribute(attribute:"plugin_publication_date", value:"2019/04/10");
    script_set_attribute(attribute:"plugin_type", value:"remote");
    script_end_attributes();
    script_summary(english: "Reports if Adminer (https://www.adminer.org/) was discovered on the remote host.");
    script_category(ACT_GATHER_INFO);
    script_copyright(english:"This script is Copyright (C) Kaimi (https://kaimi.io)");
    script_family(english: "CGI abuses");

    script_dependencie("webmirror.nasl");
    script_exclude_keys("Settings/disable_cgi_scanning");
    script_require_keys("Settings/enable_web_app_tests");
    script_require_ports("Services/www");

    script_timeout(1800);

    exit(0);
}

include("audit.inc");
include("global_settings.inc");
include("misc_func.inc");
include("http.inc");

app = "Adminer";

port = get_http_port(default: 80);

prefix_list = make_list("adminer-", "editor-", "adminer");
# Versions up to current for 4/10/2019
version_list = make_list("","4.7.1","4.7.0","4.6.3","4.6.2","4.6.1","4.6.0","4.5.0","4.4.0","4.3.1","4.3.0","4.2.5","4.2.4","4.2.3","4.2.2","4.2.1","4.2.0","4.1.0","4.0.3","4.0.2","4.0.1","4.0.0","3.7.1","3.7.0","3.6.4","3.6.3","3.6.2","3.6.1","3.6.0","3.5.1","3.5.0","3.4.0","3.3.4","3.3.3","3.3.1","3.3.0","3.2.2","3.2.1","3.2.0","3.1.0","3.0.1","3.0.0");
# More languages are available according to https://github.com/vrana/adminer
suffix_list = make_list("-en.php", "-mysql-en.php", "-mysql.php", ".php", "/");

found_list = make_list();
found_list_ver = make_list();
found_ctr = 0;
high_severity = 0;

dirs = list_uniq(make_list(cgi_dirs(), ""));

foreach dir (dirs)
{
    path = dir + '/';
    foreach prefix (prefix_list)
    {
        foreach version (version_list)
        {
            foreach suffix (suffix_list)
            {
                # path/ + adminer- + 4.1.0 + -en.php
                url = path + prefix + version + suffix;
                res = http_send_recv3(
                    method : "GET",
                    port : port,
                    item : url
                );
                if (isnull(res))
                    continue;
    
                match = pregmatch(pattern: '<span class="version">([0-9\\.]+)</span>', string: res[2], icase: TRUE);
                if (!empty_or_null(match))
                {
                    # LOAD DATA LOCAL INFILE disabled since 4.6.3
                    if (match[1] =~ "^(?:[1-4]\.[0-6]\.[0-2]|[1-3]\.[0-9]\.[0-9])$")
                        high_severity = 1;
    
                    found_list[found_ctr] = url;
                    found_list_ver[found_ctr] = match[1];
    
                    found_ctr++;
                }
            }
        }
    }
}

if (found_ctr > 0)
{
    report = NULL;
    if (report_verbosity > 0)
    {
        report += '\nThe following instance of Adminer was detected on the remote host: \n';
        report += '\n';
        for (i = 0; i < found_ctr; i++)
        {
            url = found_list[i];
            version = found_list_ver[i];

            report += 'Adminer Version\t: ' + version + '\n';
            report += 'URL\t\t: ' + build_url(port: port, qs: url) + '\n';
            report += '\n';
        }

        if (high_severity)
            report += '\nDiscovered versions may be susceptible to LOAD DATA LOCAL INFILE attack\n';
    }

    if (high_severity)
        security_hole(port: port, extra: report);
    else
        security_note(port: port, extra: report);
}
else
{
    audit(AUDIT_WEB_APP_NOT_AFFECTED, app, build_url(port: port, qs:"/"));
}